package cn.com.ty.lift.system.user.service.impl;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.system.user.dao.entity.UserAccount;
import cn.com.ty.lift.system.user.dao.entity.UserInfo;
import cn.com.ty.lift.system.user.dao.entity.model.UserRequest;
import cn.com.ty.lift.system.user.service.IUserAccountService;
import cn.com.ty.lift.system.user.service.IUserInfoService;
import cn.com.ty.lift.system.user.service.IUserService;
import cn.com.ty.lift.system.utils.PasswordUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * @author huangyuan
 * @date 2019-11-29
 * @description 用户接口
 */
@Service
public class UserService implements IUserService {

    @Autowired
    private IUserAccountService userAccountService;

    @Autowired
    private IUserInfoService userInfoService;

    @Override
    public UserAccount saveUserInfo(UserRequest userRequest) {
        String password = userRequest.getPassword();
        UserAccount userAccount = new UserAccount();
        userAccount.setMobile(userRequest.getMobile());
        //设置账号(小梯号)
        userAccount.setAccount(userRequest.getAccount());
        userAccount.setCreateDate(new Date());
        //设置用户类型
        userAccount.setType(ApiConstants.UserConstants.TYPE_USER);
        UserInfo userInfo = new UserInfo();
        userInfo.setName(userRequest.getName());

        //前端传递MD5加密密码，然后进行密码加密
        //1.生成16位随机码
        String salt = PasswordUtils.randomCode();
        //2.对密码进行加盐处理，并对加盐的密码进行哈希加密
        password = PasswordUtils.generatePassword(password, salt);
        userAccount.setSalt(salt);
        userAccount.setPassword(password);

        //保存用户账户
        boolean insertAccountFlag = userAccountService.save(userAccount);
        //保存用户信息
        userInfo.setUserId(userAccount.getUserId());
        boolean insertInfoFlag = userInfoService.save(userInfo);
        if (!insertAccountFlag || !insertInfoFlag) {
            return null;
        }
        return userAccount;
    }

    @Override
    public IPage<UserAccount> list(UserRequest userRequest) {
        IPage<UserAccount> userAccountPage = new Page<>(userRequest.getPageNum(), userRequest.getPageSize());
        userAccountPage = userAccountService.page(userAccountPage, new QueryWrapper<UserAccount>()
                //通过用户名模糊查询
                .like(StringUtils.isNotBlank(userRequest.getQueryCondition()), "account", userRequest.getQueryCondition())
                .or()
                //通过用户手机号模糊查询
                .like(StringUtils.isNotBlank(userRequest.getQueryCondition()), "mobile", userRequest.getQueryCondition())
                .eq("delete_flag", ApiConstants.DELETE_NO)
        );
        return userAccountPage;
    }

    @Override
    @Transactional
    public RestResponse addUser(UserRequest userRequest) {
        //保存用户信息，成功返回用户账户信息，失败返回空
        UserAccount userAccount = this.saveUserInfo(userRequest);
        if (userAccount == null) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, "新增用户失败");
        }
        return RestResponse.ok(userAccount, ApiConstants.RESULT_SUCCESS, "新增用户成功");
    }

    @Override
    @Transactional
    public RestResponse updateUser(UserRequest userRequest) {
        UserAccount userAccount = userAccountService.getByUserId(userRequest.getUserId());
        UserInfo userInfo = userInfoService.getByUserId(userRequest.getUserId());
        userAccount.setMobile(userRequest.getMobile());
        userAccount.setAccount(userRequest.getAccount());
        userInfo.setName(userRequest.getName());
        boolean updateAccountFlag = userAccountService.updateUserAccount(userAccount);
        boolean updateInfoFlag = userInfoService.updateUserInfo(userInfo);
        if (!updateAccountFlag || !updateInfoFlag) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, "更新用户信息失败");
        }
        return RestResponse.ok(null, ApiConstants.RESULT_SUCCESS, "更新用户信息成功");
    }

    @Override
    @Transactional
    public RestResponse deleteUser(UserRequest userRequest) {
        UserAccount userAccount = userAccountService.getByUserId(userRequest.getUserId());
        UserInfo userInfo = userInfoService.getByUserId(userRequest.getUserId());
        if (userAccount != null && userInfo != null) {
            userAccount.setDeleteFlag(ApiConstants.DELETE_YES);
            userInfo.setDeleteFlag(ApiConstants.DELETE_YES);
            boolean updateAccountFlag = userAccountService.updateUserAccount(userAccount);
            boolean updateInfoFlag = userInfoService.updateUserInfo(userInfo);
            if(updateAccountFlag && updateInfoFlag){
                return RestResponse.ok(null, ApiConstants.RESULT_SUCCESS, "删除用户成功");
            }
        }
        return RestResponse.error(ApiConstants.RESULT_ERROR, "用户不存在无法删除");
    }

}
